@@ -553,6 +553,9 @@ void configureApplication()
   QCoreApplication::setOrganizationDomain(GMIC_QT_ORGANISATION_DOMAIN);
   QCoreApplication::setApplicationName(GMIC_QT_APPLICATION_NAME);
   QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuBar);
+  if (GMIC_SETTINGS_INLINE.value(HIGHDPI_KEY, false).toBool()) {
+    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+  }
 #endif
 }
 
